/* nurbies.c -- nurbies visualization plugin for xmms 
   Written by Bryan Rittmeyer <bryanr@caltech.edu>
   Copyright (C) 1999,2000 Horizon Technologies
   http://horizon.pair.com/
   
   Fixes by George Kyriazis (kyriazis@nvidia.com>

   I did it all for the nurbie
   so you can take that furby...                      */

#include "nurbies.h"

/* external plugin functions */
static void nurbies_init(void);
static void nurbies_cleanup(void);
static void nurbies_playback_start(void);
static void nurbies_playback_stop(void);
static void nurbies_render_pcm(gint16 data[2][512]);
static void nurbies_about(void);

/* Define plugin */
VisPlugin nurbies_vp =
{
  NULL,
  NULL,
  0,
  "Nurbies",
  1,
  0,
  nurbies_init,
  nurbies_cleanup,
  nurbies_about,
  plugin_configure,
  NULL,
  nurbies_playback_start,
  nurbies_playback_stop,
  nurbies_render_pcm,
  NULL
};

/* return visplugin info for xmms */
VisPlugin *get_vplugin_info(void)
{
  return &nurbies_vp;
}


void plugin_read_config(void)
{
	ConfigFile *cfg;	
	gchar *filename;

	plugin_cfg.tdfx_mode = FALSE;
	
	filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
	cfg = xmms_cfg_open_file(filename);
	
	if (cfg)
	{
		xmms_cfg_read_boolean(cfg, "Nurbies", "tdfx_fullscreen",
				      &plugin_cfg.tdfx_mode);		
		xmms_cfg_free(cfg);
	}
	g_free(filename);
}

/* initialize plugin */
static void nurbies_init(void)
{
  plugin_read_config();
  if(!plugin_cfg.tdfx_mode)
    start_display();

/*   seconds=time(NULL); */
/*   srand(seconds); */
}

/* shut down plugin */
static void nurbies_cleanup(void)
{
  stop_display();
}

/* begin playback */
static void nurbies_playback_start(void)
{
/*   create_nurbies(); */
  /*  going=TRUE; */
  if(plugin_cfg.tdfx_mode) {
    if(window)
      stop_display();
    start_display();
  }
}

/* stop playback */
static void nurbies_playback_stop(void)
{
  /*   create_nurbies(); */
  /*  going=FALSE; */
  if(plugin_cfg.tdfx_mode) {
    stop_display();
  }
}

/* render visualization frame */
static void nurbies_render_pcm(gint16 data[2][512])
{
  int i,j;
  double total[8];

  /* waveform analysis */
  for(i=0;i<512;i+=64) {
    total[i/64]=0.0f;
    for(j=i;j<(i+64); j++)
      total[i/64]+=(data[0][j]>>7);
    total[i/64]/=64.0f;
    total[i/64]/=56.0f;
    total[i/64]-=1.5f;
  }
    
    
  /* background calculations */
  col+=(768/360);
  if(col>=768)
    col=0;

  if(col<256) {
    r=255-col;
    g=col;
    b=0;
  }
  if(col>255 && col<512) {
    r=0;
    g=512-col;
    b=col-256;
  }
  if(col>511) {
    r=col-512;
    g=0;
    b=768-col;
  }

  /* calculate nurbies */
  for (u = 0; u < 4; u++)
    {
      for (v = 0; v < 4; v++)
	{
	  /* red nurbie */
	  pts1[u][v][0] = 2.0 * ((GLfloat)u);
	  pts1[u][v][1] = 2.0 * ((GLfloat)v);
	  
	  if((u==1 || u == 2) && (v == 1 || v == 2))
	    pts1[u][v][2] = 6.0 + total[0];
	  else
	    pts1[u][v][2] = 0.0;
	  
	  /* green nurbie */
	  pts2[u][v][0] = 2.0 * ((GLfloat)u - 3.0);
	  pts2[u][v][1] = 2.0 * ((GLfloat)v - 3.0);
	  
	  if((u==1 || u == 2) && (v == 1 || v == 2))
	    {
	      if(u == 1 && v == 1) 
		pts2[u][v][2] = 10.0 + (total[3] * 2);
	      else
		pts2[u][v][2] = -2.0 + (total[4] * 4);
	    }
	  else
	    pts2[u][v][2] = 0.0;
	  
	  /* blue nurbie */
	  pts3[u][v][0] = 2.0 * ((GLfloat)u - 3.0);
	  pts3[u][v][1] = 2.0 * ((GLfloat)v);
	  
	  if((u==1 || u == 2) && (v == 1 || v == 2))
	    {
	      if(u == 1 && v == 2)
		pts3[u][v][2] = 11.0 + (total[5] * 2);
	      else
		pts3[u][v][2] = 2.0 + (total[6] * 2);
	    }
	  else
	    pts3[u][v][2] = 0.0;
	  
	  /* yellow nurbie */
	  pts4[u][v][0] = 2.0 * ((GLfloat)u);
	  pts4[u][v][1] = 2.0 * ((GLfloat)v - 3.0);
	  
	  if((u==1 || u == 2 || u == 3) && (v == 1 || v == 2))
	    {
	      if(v == 1)
		pts4[u][v][2] = -2.0 + (total[7] * 2);
	      else
		pts4[u][v][2] = 5.0 + total[7];
	    }
	  else
	    pts4[u][v][2] = 0.0;
	}
    }
  
  /* stretch up red's far right corner */
  pts1[3][3][2] = 3 + total[1];
  
  /* Pull down green's near left corner a little */
  pts2[0][0][2] = -2 + total[2];
  
  /* Turn up meeting of four corners */
  pts1[0][0][2] = 1;
  pts2[3][3][2] = 1;
  pts3[3][0][2] = 1;
  pts4[0][3][2] = 1;

}

/* about box */
static void nurbies_about(void)
{

  dialog = gtk_dialog_new();
  gtk_window_set_title(GTK_WINDOW(dialog), "About Nurbies");
  gtk_container_border_width(GTK_CONTAINER(dialog), 5);
  label = gtk_label_new("Nurbies Visualization Plugin for xmms\n \
Written by Bryan Rittmeyer <bryanr@caltech.edu>\n \
Copyright (C) 1999 Horizon Technologies\n \
http://horizon.pair.com/\n\n \
This program is free software; you can redistribute it and/or modify\n\
it under the terms of the GNU General Public License as published by\n\
the Free Software Foundation; either version 2 of the License, or\n\
(at your option) any later version.\n\
\n\
This program is distributed in the hope that it will be useful,\n\
but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
GNU General Public License for more details.\n\
\n\
You should have received a copy of the GNU General Public License\n\
along with this program; if not, write to the Free Software\n\
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,\n\
USA.");
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
  gtk_widget_show(label);
  
  button = gtk_button_new_with_label(" Close ");
  gtk_signal_connect_object(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(dialog));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, FALSE, FALSE, 0);
  gtk_widget_show(button);

  gtk_widget_show(dialog);
  gtk_widget_grab_focus(button);
}

static Window create_window(int width, int height)
{
  int attr_list[] = { GLX_RGBA,
		      GLX_DEPTH_SIZE, 16, 
		      GLX_DOUBLEBUFFER,
		      None };
  int scrnum;
  XSetWindowAttributes attr;
  unsigned long mask;
  Window root, win;
  XVisualInfo *visinfo;
  Atom wm_protocols[1];
  
  scrnum = DefaultScreen(dpy);
  root = RootWindow(dpy, scrnum);

  visinfo = glXChooseVisual(dpy, scrnum, attr_list);
  if(!visinfo)
     return 0;

  attr.background_pixel = 0;
  attr.border_pixel = 0;
  attr.colormap = colormap = XCreateColormap(dpy, root, visinfo->visual, AllocNone);
  attr.event_mask = StructureNotifyMask | KeyPressMask;
  mask = CWBackPixel | CWBorderPixel | CWColormap | CWEventMask;

  win = XCreateWindow(dpy, root, 0, 0, width, height,
                      0, visinfo->depth, InputOutput,
                      visinfo->visual, mask, &attr);
  XmbSetWMProperties(dpy, win, "Nurbies",
                     "Nurbies", NULL, 0, NULL, NULL,
			   NULL);
  wm_delete_window_atom = wm_protocols[0] = XInternAtom(dpy,
							"WM_DELETE_WINDOW",
							 False);
  XSetWMProtocols(dpy, win, wm_protocols, 1);

  glxcontext = glXCreateContext(dpy, visinfo, NULL, True);

  glXMakeCurrent(dpy, win, glxcontext);

  return win;
}


void *draw_thread_func(void *arg)
{
  Bool configured=FALSE;
  
  #ifdef HAVE_XMESAFINDBUFFER
  XMesaBuffer xmesabuffer;
#endif


#ifdef HAVE_SCHED_SETSCHEDULER
  if(xmms_check_realtime_priority()) {
    struct sched_param sparam;
    sparam.sched_priority = sched_get_priority_max(SCHED_OTHER);
    pthread_setschedparam(pthread_self(), SCHED_OTHER, &sparam);
  }
#endif

  if(plugin_cfg.tdfx_mode)
    setenv("MESA_GLX_FX", "fullscreen", TRUE);
  else
    unsetenv("MESA_GLX_FX");

  dpy=XOpenDisplay(NULL);
  window=create_window(640,480);
  XMapWindow(dpy, window);

  /* Setup nurbies */
  glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_AUTO_NORMAL);
  glEnable(GL_NORMALIZE);
  nurb = gluNewNurbsRenderer();
  gluNurbsProperty(nurb, GLU_SAMPLING_TOLERANCE, 25.0);
  gluNurbsProperty(nurb, GLU_DISPLAY_MODE, GLU_FILL);

  glMatrixMode(GL_PROJECTION);
  gluPerspective(55.0, 1.0, 2.0, 24.0);
  glMatrixMode(GL_MODELVIEW);
  glTranslatef(0.0, 0.0, -15.0);
  glRotatef(330.0, 1.0, 0.0, 0.0);

  create_nurbies();
  going=TRUE;

  while(going)
    {
      while(XPending(dpy))
	{
	  XEvent event;
	  KeySym keysym;
	  char buf[16];

	  XNextEvent(dpy,&event);
	  switch(event.type)
	    {
	    case ConfigureNotify:
	      glViewport(0,0,event.xconfigure.width,event.xconfigure.height);
	      if(plugin_cfg.tdfx_mode && !grabbed_pointer) {
		XGrabPointer(dpy, window,
			     True, ButtonPressMask,
			     GrabModeAsync,
			     GrabModeAsync,
			     window, None, CurrentTime);
		grabbed_pointer = TRUE;
	      }
	      configured=TRUE;
	      break;
	    case KeyPress:
	      XLookupString(&event.xkey,buf,16,&keysym,NULL);
	      switch(keysym)
		{
		case XK_Escape:
		  GDK_THREADS_ENTER();
		  gtk_idle_add(disable_func,NULL);
		  GDK_THREADS_LEAVE();
		  break;
		case XK_z:
		  xmms_remote_playlist_prev(nurbies_vp.xmms_session);
		case XK_x:
		  xmms_remote_play(nurbies_vp.xmms_session);
		  break;
		case XK_c:
		  xmms_remote_pause(nurbies_vp.xmms_session);
		  break;
		case XK_v:
		  xmms_remote_stop(nurbies_vp.xmms_session);
		  break;
		case XK_b:
		  xmms_remote_playlist_next(nurbies_vp.xmms_session);
		  break;
		case XK_Up:
		  yrot-=0.1;
		  if(yrot<-3.0)
		    yrot=-3.0;
		  break;
		case XK_Down:
		  yrot+=0.1;
		  if(yrot>3.0)
		    yrot=3.0;
		  break;
		case XK_Left:
		  xrot-=0.1;
		  if(xrot<-3.0)
		    xrot=-3.0;
		  break;
		case XK_Right:
		  xrot+=0.1;
		  if(xrot>3.0)
		    xrot=3.0;
		  break;
		}
	      break;
	    case ClientMessage: /* Not the X! Anything but that! */
	      if ((Atom)event.xclient.data.l[0] == wm_delete_window_atom) {
		  GDK_THREADS_ENTER();
		  gtk_idle_add(disable_func, NULL);
		  GDK_THREADS_LEAVE();
		}
	      break;
	    }
	}
      if(configured) {
	draw_nurbies();
      }
    }

  if(glxcontext)
    {
      glXMakeCurrent(dpy, 0, NULL);
      glXDestroyContext(dpy, glxcontext);
      glxcontext = NULL;
#ifdef HAVE_XMESAFINDBUFFER
      xmesabuffer = XMesaFindBuffer(dpy, window);
      if(xmesabuffer)
	XMesaDestroyBuffer(xmesabuffer);
#endif
    }	
  if(window)
    {
      if(grabbed_pointer)
	{
	  XUngrabPointer(dpy, CurrentTime);
	  grabbed_pointer = FALSE;
	}
      
      XDestroyWindow(dpy, window);
      window = 0;
    }
  if(colormap)
    {
      XFreeColormap(dpy, colormap);
      colormap = 0;
    }
  if(dpy)
    {
      XCloseDisplay(dpy);
      dpy = NULL;
    }

  pthread_exit(NULL);
}

static gint disable_func(gpointer data)
{
  nurbies_vp.disable_plugin(&nurbies_vp);
  return FALSE;
}

static void start_display(void)
{
  pthread_create(&draw_thread,NULL,draw_thread_func,NULL);
}

static void stop_display(void)
{

  if(going)
    {
      going = FALSE;
      pthread_join(draw_thread, NULL);
    }
}


void create_nurbies(void)
{
  /* Build control points for NURBS mole hills. */
  for(u=0; u<4; u++) {
    for(v=0; v<4; v++) {
      /* Red. */
      pts1[u][v][0] = 2*((GLfloat)u);
      pts1[u][v][1] = 2*((GLfloat)v);
      pts1[u][v][2] = 0.0;

      /* Green. */
      pts2[u][v][0] = 2.0*((GLfloat)u - 3.0);
      pts2[u][v][1] = 2.0*((GLfloat)v - 3.0);
      pts2[u][v][2] = 0.0;
      
      /* Blue. */
      pts3[u][v][0] = 2.0*((GLfloat)u - 3.0);
      pts3[u][v][1] = 2.0*((GLfloat)v);
      pts3[u][v][2] = 0.0;
      
      /* Yellow. */
      pts4[u][v][0] = 2.0*((GLfloat)u);
      pts4[u][v][1] = 2.0*((GLfloat)v - 3.0);
      pts4[u][v][2] = 0.0;
    }
  }
  /* Stretch up red's far right corner. */
/*   pts1[3][3][2] = rand()%6; */
  /* Pull down green's near left corner a little. */
/*   pts2[0][0][2] = -rand()%2; */
  /* Turn up meeting of four corners. */
  pts1[0][0][2] = 1;
  pts2[3][3][2] = 1;
  pts3[3][0][2] = 1;
  pts4[0][3][2] = 1;

}

static void draw_nurbies(void)
{
  glNewList(1, GL_COMPILE);

  /* Render red hill. */
  glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_red_diffuse);
  gluBeginSurface(nurb);
  gluNurbsSurface(nurb, 8, knots, 8, knots,
		  4 * 3, 3, &pts1[0][0][0],
		  4, 4, GL_MAP2_VERTEX_3);
  gluEndSurface(nurb);

  /* Render green hill. */
  glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_green_diffuse);
  gluBeginSurface(nurb);
  gluNurbsSurface(nurb, 8, knots, 8, knots,
		  4 * 3, 3, &pts2[0][0][0],
		  4, 4, GL_MAP2_VERTEX_3);
  gluEndSurface(nurb);
  
  /* Render blue hill. */
  glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_blue_diffuse);
  gluBeginSurface(nurb);
  gluNurbsSurface(nurb, 8, knots, 8, knots,
		  4 * 3, 3, &pts3[0][0][0],
		  4, 4, GL_MAP2_VERTEX_3);
  gluEndSurface(nurb);
  
  /* Render yellow hill. */
  glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_yellow_diffuse);
  gluBeginSurface(nurb);
  gluNurbsSurface(nurb, 8, knots, 8, knots,
		  4 * 3, 3, &pts4[0][0][0],
		  4, 4, GL_MAP2_VERTEX_3);
  gluEndSurface(nurb);
  glEndList();

  glClearColor(r/256.0f, g/256.0f, b/256.0f, 1.0f);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glRotatef(xrot,yrot,0.0f,1.0f);
  glCallList(1);
  glXSwapBuffers(dpy,window); /* Swap those buffers */
}
